package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Bandbox;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.enums.StatusRecargaEnum;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioRecargaRvhub;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderRelatorioVendasBilheteiro;

@Controller("relatorioRecargaRvhubController")
@Scope("prototype")
public class RelatorioRecargaRvhubController extends MyGenericForwardComposer {

	private static final String TITULO = "relatorioRecargaRvhubController.window.title";

	private static final long serialVersionUID = 1L;

	@Autowired
	private DataSource dataSourceRead;

	@Autowired
	private EmpresaService empresaService;
	
	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;

	private Datebox datInicial;
	private Datebox datFinal;
	private MyComboboxEstandar cmbEmpresa;	
	private List<Empresa> lsEmpresa;	
	
	private MyTextbox txtNombrePuntoVenta;
	private Bandbox bbPesquisaPuntoVenta;
	private MyListbox puntoVentaList;
	private MyListbox puntoVentaSelList;
	private Paging pagingPuntoVenta;
	private Checkbox chkAutorizado;
	private Checkbox chkCancelado;
	private Checkbox chkConfirmado;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();		
		super.doAfterCompose(comp);

		puntoVentaList.setItemRenderer(new RenderRelatorioVendasBilheteiro());

	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	private void executarPesquisaAgencia() {
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda = new HibernateSearchObject<>(PuntoVenta.class, pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterILike("nombpuntoventa", "%" + txtNombrePuntoVenta.getValue() + "%");
		puntoVentaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		puntoVentaBusqueda.addSortAsc("nombpuntoventa");

		puntoVentaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel(TITULO),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
				ex.printStackTrace();
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) {
		executarPesquisaAgencia();
	}

	public void onDoubleClick$puntoVentaSelList(Event ev) {
		PuntoVenta puntoVenta = (PuntoVenta) puntoVentaSelList.getSelected();
		puntoVentaSelList.removeItem(puntoVenta);
	}

	public void onDoubleClick$puntoVentaList(Event ev) {
		PuntoVenta puntoVenta = (PuntoVenta) puntoVentaList.getSelected();
		puntoVentaSelList.addItemNovo(puntoVenta);
	}

	public void onClick$btnLimpar(Event ev) {
		puntoVentaList.setData(new ArrayList<PuntoVenta>());

		bbPesquisaPuntoVenta.setText("");
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void executarRelatorio() throws Exception {
		Relatorio relatorio;
		Map<String, Object> parametros = new HashMap<String, Object>();
		StringBuilder filtro = new StringBuilder();
		
		if (this.datInicial.getValue() != null) {
			parametros.put("datInicial", this.datInicial.getValue());
		}

		if (this.datFinal.getValue() != null) {
			parametros.put("datFinal", this.datFinal.getValue());
		}
		
		if (parametros.get("datInicial") == null && parametros.get("datFinal") == null) {
			Messagebox.show(Labels.getLabel("MSG.Error.dataObrigatoria"),
					Labels.getLabel(TITULO),
					Messagebox.OK, Messagebox.INFORMATION);
			return;
		}
		
		filtro.append("Período: ");		
		filtro.append( DateUtil.getStringDate(datInicial.getValue()));

		filtro.append(" - ");
		filtro.append( DateUtil.getStringDate(datFinal.getValue()));
		
		filtro.append("; Empresa: ");
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			parametros.put("EMPRESA_ID", empresa.getEmpresaId());
			filtro.append(empresa.getNombempresa() + ";");
		} else {
			filtro.append(" Todas;");
		}

		filtro.append(" Agência: ");
		String puntoVentaIds = "";
		String puntoVentas = "";
		List<PuntoVenta> lsPuntoVentaSelecionados = new ArrayList(Arrays.asList(puntoVentaSelList.getData()));
		if (lsPuntoVentaSelecionados.isEmpty()) {
			puntoVentas = "Todas";
		} else {
			for (int i = 0; i < lsPuntoVentaSelecionados.size(); i++) {
				PuntoVenta puntoVenta = lsPuntoVentaSelecionados.get(i);
				puntoVentas = puntoVentas + puntoVenta.getNombpuntoventa() + ",";

				puntoVentaIds = puntoVentaIds + puntoVenta.getPuntoventaId() + ",";
			}

			// removendo ultima virgula
			puntoVentaIds = puntoVentaIds.substring(0, puntoVentaIds.length() - 1);
			puntoVentas = puntoVentas.substring(0, puntoVentas.length() - 1);
			parametros.put("NUMPUNTOVENTA", puntoVentaIds);
		}		
		
		filtro.append(puntoVentas).append(";");

		parametros.put("NOME_RELATORIO", Labels.getLabel(TITULO));
		parametros.put("USUARIO", UsuarioLogado.getUsuarioLogado().getNombusuario());

		StringBuilder status = new StringBuilder();
		StringBuilder statusDesc = new StringBuilder();
		if (chkCancelado.isChecked()) {			
			status.append("'" )
			.append( StatusRecargaEnum.CANCELADA )
			.append( "',");
			statusDesc.append("Cancelado,");
		}
		
		if (chkAutorizado.isChecked()) {
			status.append("'" )
			.append( StatusRecargaEnum.AUTORIZADA )
			.append( "',");
			statusDesc.append("Autorizado,");
		}
		
		if (chkConfirmado.isChecked()) {
			status.append("'" )
			.append( StatusRecargaEnum.CONFIRMADA )
			.append( "',");
			statusDesc.append("Confirmado,");
		}
		
		if (status.length() > 0) {
			status.deleteCharAt(status.length() - 1);
			statusDesc.deleteCharAt(statusDesc.length() - 1);
			filtro.append(" Status: ").append(statusDesc);
			parametros.put("STATUS", status);
		}

		relatorio = new RelatorioRecargaRvhub(parametros, dataSourceRead.getConnection());		
		parametros.put("FILTROS", filtro.toString());

		Map args = new HashMap();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul", Labels.getLabel(TITULO), args, MODAL);
	}
}
